/* SiME Heap String functions */

/* Copyright 2021 Kazuo K.
 * SiME is licensed under the MIT license. You should have
 * received a copy of the license with your distribution.
 */

#include <stdarg.h>

#include <X11/Intrinsic.h>

#include "heap_string.h"


char  *Xstrdup(char *str)
{
   char *p = XtMalloc((strlen(str) + 1) * sizeof(char));
   strcpy(p, str);
   return(p);
}

char  *Xstrndup(char *str, int n)
{
   char *p = XtMalloc((n + 1) * sizeof(char));
   strncpy(p, str, n);
   p[n] = '\0';
   return(p);
}


#define  DS_STEP  100

void heap_string_init(hstring *ds)
{
   ds->str = NULL;
   ds->len = 0;
}

void heap_string_free(hstring *ds)
{
   XtFree(ds->str);
   heap_string_init(ds);
}


static void heap_string_resize(hstring *ds, int len)
{
   if (len > ds->len)
   {
      len = ((len + DS_STEP - 1) / DS_STEP) * DS_STEP;
      if (!ds->str)
      {
         ds->str = XtMalloc(len * sizeof(char));
         ds->str[0] = '\0';
      }
      else
         ds->str = XtRealloc(ds->str, len * sizeof(char));
      ds->len = len;
   }
}


void heap_string_new(hstring *ds, char *str)
{
   heap_string_init(ds);
   heap_string_cat(ds, str);
}

void heap_string_compose(hstring *ds, ...)
{
   va_list  vl;
   char    *p;

   heap_string_init(ds);
   va_start(vl, ds);
   while((p = va_arg(vl, char*)) != NULL)
      heap_string_cat(ds, p);
   va_end(vl);
}

void heap_string_cat(hstring *ds, char *str)
{
   int  akt_len = ds->str ? strlen(ds->str) : 0;
   int  new_len = akt_len + strlen(str) + 1;
   heap_string_resize(ds, new_len);
   strcat(ds->str, str);
}

void heap_strncat(hstring *ds, char *str, int n)
{
   int  akt_len = ds->str ? strlen(ds->str) : 0;
   int  new_len = akt_len + n + 1;
   heap_string_resize(ds, new_len);
   strncat(ds->str, str, n);
}
