/* SiME Help Events*/

/* Copyright 2021 Kazuo K.
 * SiME is licensed under the MIT license. You should have
 * received a copy of the license with your distribution.
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <errno.h>

#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/List.h>

#include "sys_consts.h"
#include "globals.h"
#include "utils.h"
#include "callbacks.h"
#include "pop_ups.h"
#include "helpevents.h"
#include "heap_string.h"


static Boolean load_help_file(char *filename);

/* Main help func */
void  gui_help_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   XmString item;

   set_status_message("Ok, running Help");

   /* Determine how help_CB was called */
   switch ((HelpReason)client_data)
   {
      case HELPCALL:
	 if (helpPopUp == (Widget)0)
	    gui_create_help_popup(toplevel);
	 XtManageChild(helpPopUp);
	 break;

      case HELPCANCEL:
	 XtUnmanageChild(helpPopUp);
	 set_status_message("");
	 break;

      case HELPHELP:	
	 item = XmStringCreateLtoR("helpOnHelp", CHARSET);
	 XmListSelectItem(helpLI, item, True);
	 break;

      case HELPSETUP:		
	 if (helpPopUp == (Widget) 0)
	    gui_create_help_popup(toplevel);
	 XtManageChild(helpPopUp);
	 item = XmStringCreateLtoR("SetUp", CHARSET);
	 XmListSelectItem(helpLI, item, True);
	 break;

      case HELPABOUT:
	 gui_about();
	 break;

      default:
	 break;
   }
}

void  gui_help_sel_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   XmListCallbackStruct *cbd = (XmListCallbackStruct *)call_data;
   char    *curr_item;

   XmStringGetLtoR(cbd->item, CHARSET, &curr_item);
   load_help_file(curr_item);
   XtFree(curr_item);
}


void  gui_create_helplist(void)
{
   XmString item;
   struct stat stat_val;
   char    *filecontents;
   FILE    *fpointer;
   int      f_length;
   char    *idstring;
   hstring  filename;
   int      count;

   filecontents = NULL;
   heap_string_compose(&filename, HELPPATH, "/SiMEHelp", NULL);
   if (stat(filename.str, &stat_val) == 0)
   {
      f_length = stat_val.st_size;
      fpointer = fopen(filename.str, "r");
      if (fpointer != NULL)
      {
	 filecontents = XtMalloc((unsigned)(f_length + 1));
	 *filecontents = '\0';
	 fread(filecontents, sizeof(char), f_length, fpointer);
	 filecontents[f_length] = '\0';
	 fclose(fpointer);
      }
      if (filecontents != NULL)
      {
	 count = 1;
	 XmListDeselectAllItems(helpLI);
	 idstring = strtok(filecontents, SEPCHARS);
	 while (idstring != NULL)
	 {
	    item = XmStringCreateLtoR(idstring, CHARSET);
	    XmListAddItemUnselected(helpLI, item, count);
	    idstring = strtok(NULL, SEPCHARS);
	    count++;
	 };
	 XmListSelectPos(helpLI, 1, True);
	 XtFree(filecontents);
      }
   }
   else
      gui_error("Unfortunately the help data is missing or corrupted.");
   heap_string_free(&filename);
}

static Boolean load_help_file(char *filename)
{
   struct stat stat_val;
   char    *filecontents;
   FILE    *fpointer;
   int      f_length;
   hstring  file;
   Boolean  result = False;

   heap_string_compose(&file, HELPPATH, "/", filename, NULL);
   if (stat(file.str, &stat_val) == 0)
   {
      f_length = stat_val.st_size;
      fpointer = fopen(file.str, "r");
      if (fpointer != NULL)
      {
	 filecontents = XtMalloc((unsigned)(f_length + 1));
	 *filecontents = '\0';
	 fread(filecontents, sizeof(char), f_length, fpointer);
	 filecontents[f_length] = '\0';
	 fclose(fpointer);
	 XmTextSetString(helpText, filecontents);
	 result = True;
      }
   }
   heap_string_free(&file);
   return(result);
}

void  gui_cs_help_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   char  *p;

   /* Decide what called help */
   switch ((HelpItem)client_data)
   {
      case SAVE:
	 p = "Save the active file under current name";
	 break;

      case SAVEAS:
	 p = "Save the active file under a new name";
	 break;

      case OPEN:
	 p = "Open a file";
	 break;

      case INSERT:
	 p = "Insert a text file in the current text";
	 break;

      case NEW:
	 p = "Obtain a blank document";
	 break;

      case QUIT:
	 p = "Exit SiME";
	 break;

      case COPY:
	 p = "Copy selected text to clipboard";
	 break;

      case CUT:
	 p = "Remove selected text and copy to clipboard";
	 break;

      case PASTE:
	 p = "Insert text from clipboard";
	 break;

      case UNDO:
	 p = "Undo last edit";
	 break;

      case REDO:
	 p = "Undo last undo action";
	 break;

      case SEARCH:
	 p = "Search for text string in document";
	 break;

      case REPLACE:
	 p = "Replace text string found by search";
	 break;

      case REPLACEALL:
	 p = "Replace all instances of search string with replace string";
	 break;

      case SETUP:
	 p = "Invoke set up dialog";
	 break;

      case HELPB:
	 p = "Invoke help dialog";
	 break;

      case JUMP:
	 p = "Jump to line number";
	 break;

      case JUMPTEXT:
	 p = "Enter line to jump to here";
	 break;

      case SEARCHTEXT:
	 p = "Enter search string here";
	 break;

      case REPLACETEXT:
	 p = "Enter replace string here";
	 break;

      case MATCH:
	 p = "Match parentheses characters";
	 break;

      case SELECTIONONLY:
	 p = "Set search and replace with selection only";
	 break;

      default:
         p = "";
	 break;
   }
   set_status_message(p);
}

void  gui_undo_cs_help_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   set_status_message("");
}
