
/* Copyright 2021 Kazuo K.
 * SiME is licensed under the MIT license. You should have
 * received a copy of the license with your distribution.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

#include <X11/cursorfont.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>

#include "sys_consts.h"
#include "globals.h"
#include "utils.h"
#include "undo.h"
#include "optionsevents.h"
#include "widgets.h"
#include "fileevents.h"
#include "heap_string.h"

/* Global Variables */
XtAppContext app_context;
Widget   toplevel;
char    *FileName = NULL;
Cursor   MainCursor;
Cursor   TextCursor;
Cursor   TimeCursor;
Boolean  AutoSavePossible;

char    *sime_path = "/.sime";


extern void InstallActions(void);

static void init_autosave(char *home)
{
   hstring      filename;
   struct stat  buf;
   FILE  *fp;

   heap_string_compose(&filename, home, sime_path, NULL);
   if (stat(filename.str, &buf) == -1)
      mkdir(filename.str, 0700);

   heap_string_cat(&filename, "/.testfile");
   fp = fopen(filename.str, "w");
   if (!fp)
   {
      gui_error(".sime in your home directory is unavailable or not readable. Unable to utilize autosave.");
      AutoSavePossible = False;
   }
   else
   {
      fclose(fp);
      unlink(filename.str);
      AutoSavePossible = True;
   }
   heap_string_free(&filename);
}


int  main(int argc, char **argv)
{
   char  *home;

   XtSetLanguageProc(NULL, NULL, NULL);

   SelectedOnly = False;

   set_defaults();
   load_config();

   toplevel = XtVaAppInitialize(&app_context, APP_CLASS,
			NULL, 0, &argc, argv, NULL,
			XmNiconName,       APP_CLASS,
			XmNtitle,          APP_CLASS,
			XmNresizePolicy,   XmRESIZE_GROW,
			XmNdeleteResponse, XmDO_NOTHING,
			NULL);

   InstallActions();

   MainCursor = XCreateFontCursor(XtDisplay(toplevel), XC_hand2);
   TextCursor = XCreateFontCursor(XtDisplay(toplevel), XC_xterm);
   TimeCursor = XCreateFontCursor(XtDisplay(toplevel), XC_watch);

   gui_create_Widgets(toplevel);

   XtRealizeWidget(toplevel);

   home = getenv("HOME");
   if (!home) home = "";

   if ((argc == 3) && !strcmp(argv[1], "recover"))
   {
      hstring  filename;

      heap_string_compose(&filename, home, sime_path, "/", argv[2], NULL);
      if (loadfile(filename.str, OPENIT))
      {
	 FileName = Xstrdup(argv[2]);
	 set_status_message("File has been successfully recovered.");
      }
      else
      {
         XmTextSetString(text_canvas, "");
         reset_undo_redo();
	 if (gui_error("File was not successfully recovered.") == HELP)
	    gui_info("You may not have specified a valid file name\n"
	             "Or you may not have had autosave turned on");
      }
      heap_string_free(&filename);
   }
   else if (argc > 1)
   {
      int       i, flg = 0;
      FileMode  mode = OPENIT;

      FileName = Xstrdup(argv[1]);	

      XmTextSetString(text_canvas, "");
      reset_undo_redo();

      for (i=1; i<argc; i++)	
      {
	 flg |= loadfile(argv[i], mode);
	 mode = APPENDIT;		
      }

      XmTextSetInsertionPosition(text_canvas, 0);
   }

   if (!FileName)
   {
      FileName = Xstrdup("untitled");
      XmTextSetString(text_canvas, "");
      reset_undo_redo();
   }

   init_autosave(home);

   set_title_string();
   set_status_message("");

   XtRealizeWidget(toplevel);

   set_autoindent_wordwrap();
   set_autosave();

   set_cursor(toplevel, MainCursor);
   set_cursor(text_canvas, TextCursor);

   XtAppMainLoop(app_context);
}
