#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <ctype.h>

#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>

#include "sys_consts.h"
#include "globals.h"
#include "callbacks.h"
#include "widgets.h"
#include "pop_ups.h"
#include "fileevents.h"
#include "optionsevents.h"
#include "undo.h"
#include "utils.h"
#include "heap_string.h"

/* Macros for boolean for schizo libcs */
#define INT2BOOLEAN(i) ((i) ? True : False)
#define BOOLEAN2INT(b) ((b) ? 1 : 0)

/* Globals */
int       AutoSaveTime;
Boolean   AutoSave;
Boolean   AutoBackup;
Boolean   AutoIndent;
Boolean   WordWrap;
Boolean   FileBox;
char     *DefaultPrinter;
char     *defaultPrinter = NULL;

ExtMode    *extmode = NULL;
static int  extcnt  = 0;
int         extnum  = 0;

static int      autoSaveTime;
static Boolean  autoSave;
static Boolean  autoBackup;
static Boolean  autoIndent;
static Boolean  wordWrap;
static Boolean  fileBox;

/* Local Protos */
static void show_option_values(void);
static void SetExtension(char *line);


void  set_autoindent_wordwrap(void)
{
   if (AutoIndent)
      XtOverrideTranslations(text_canvas,
	    XtParseTranslationTable("<Key>Return: newline-and-indent()\n"));
   else
      XtOverrideTranslations(text_canvas,
	    XtParseTranslationTable("<Key>Return: newline()\n"));

   XtVaSetValues(text_canvas, XmNwordWrap, WordWrap, NULL);
}

void set_autosave(void)
{
   if (AutoSavePossible && AutoSave)
      XtAppAddTimeOut(app_context, AutoSaveTime*1000, auto_save_func, NULL);

}

static void show_option_values(void)
{
   char  item[20];

   sprintf(item, "%d", AutoSaveTime);
   XmTextSetString(autosavetimeText, item);

   XtVaSetValues(autosaveTB, XmNset, AutoSave,   NULL);
   XtVaSetValues(backupTB,   XmNset, AutoBackup, NULL);
   XtVaSetValues(autoindTB,  XmNset, AutoIndent, NULL);
   XtVaSetValues(wordwrapTB, XmNset, WordWrap,   NULL);
   XtVaSetValues(fileboxTB,  XmNset, FileBox,    NULL);
}


void  gui_setup_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   autoSaveTime = AutoSaveTime;
   autoSave     = AutoSave;
   autoBackup   = AutoBackup;
   autoIndent   = AutoIndent;
   wordWrap     = WordWrap;
   fileBox      = FileBox;

   if (configPopUp == (Widget) 0)
      gui_create_config_popup(toplevel);

   show_option_values();

   XtManageChild(configPopUp);
}


void  gui_save_conf_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   FILE    *fpointer;
   char    *home;
   hstring  filename;
   int      i;

   home = getenv("HOME");
   if (!home)
      home = ".";
   heap_string_compose(&filename, home, "/.simerc", NULL);
   fpointer = fopen(filename.str, "w");
   if (fpointer != NULL)
   {
      fprintf(fpointer, "AutoSaveTime %d\n", AutoSaveTime);
      fprintf(fpointer, "AutoSave     %d\n", BOOLEAN2INT(AutoSave));
      fprintf(fpointer, "Backup       %d\n", BOOLEAN2INT(AutoBackup));
      fprintf(fpointer, "AutoIndent   %d\n", BOOLEAN2INT(AutoIndent));
      fprintf(fpointer, "WordWrap     %d\n", BOOLEAN2INT(WordWrap));
      fprintf(fpointer, "FileBox      %d\n", BOOLEAN2INT(FileBox));
      fprintf(fpointer, "\n");
      fprintf(fpointer, "# Extension WordWrap AutoIndent\n");
      for (i=0; i<extnum; i++)
         fprintf(fpointer, "*.%s\t\t%d\t%d\n", extmode[i].ext, extmode[i].ww, extmode[i].ai);
      fclose(fpointer);
   }
   heap_string_free(&filename);
   return;
}

void  set_defaults(void)
{
   AutoSaveTime = AUTO_SAVE_TIME;
   AutoSave     = True;
   AutoBackup   = True;
   AutoIndent   = True;
   WordWrap     = False;
   FileBox      = True;
}


static void SetExtension(char *line)
{
   int  i;

   if (extnum >= extcnt)
   {
      extcnt += 10;
      extmode = (ExtMode*)XtRealloc((char*)extmode, extcnt * sizeof(ExtMode));
   }
   for (i=0; (i<15) && !isspace(line[i]); i++)
      extmode[extnum].ext[i] = line[i];
   extmode[extnum].ext[i] = '\0';
   line += i;

   extmode[extnum].ww = INT2BOOLEAN(strtol(line, &line, 10));
   extmode[extnum].ai = INT2BOOLEAN(strtol(line, &line, 10));

   extnum++;
}

Boolean  load_config(void)
{
   FILE    *fpointer;
   char    *home;
   hstring  filename;
   Boolean  result;

   home = getenv("HOME");
   if (!home) home = ".";
   heap_string_compose(&filename, home, "/.simerc", NULL);

   fpointer = fopen(filename.str, "rt");
   if (fpointer != NULL)
   {
      char  line[256];

      while(fgets(line, sizeof(line), fpointer))
      {
         char *p = line;
         while (*p && isspace(*p))
            ++p;
         if (!strncasecmp(p, "autosavetime", 12))
            AutoSaveTime = atoi(p+12);
         else if (!strncasecmp(p, "autosave", 8))
            AutoSave = INT2BOOLEAN(atoi(p+8));
         else if (!strncasecmp(p, "backup", 6))
            AutoBackup = INT2BOOLEAN(atoi(p+6));
         else if (!strncasecmp(p, "autoindent", 10))
            AutoIndent = INT2BOOLEAN(atoi(p+10));
         else if (!strncasecmp(p, "wordwrap", 8))
            WordWrap = INT2BOOLEAN(atoi(p+8));
         else if (!strncasecmp(p, "filebox", 7))
            FileBox = INT2BOOLEAN(atoi(p+7));
         else if (!strncmp(p, "*.", 2))
            SetExtension(p+2);
      }

      fclose(fpointer);

      if (AutoSaveTime < MIN_AUTO_SAVE_TIME)
	 AutoSaveTime = MIN_AUTO_SAVE_TIME;

      result = True;
   }
   else
      result = False;

   heap_string_free(&filename);

   return (result);
}

void  gui_restore_conf_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   set_defaults();
   load_config();
   show_option_values();
}

void  gui_cancel_conf_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   AutoSaveTime = autoSaveTime;
   AutoSave     = autoSave;
   AutoBackup   = autoBackup;
   AutoIndent   = autoIndent;
   WordWrap     = wordWrap;
   FileBox      = fileBox;
   strcpy(DefaultPrinter, defaultPrinter);

   XtUnmanageChild(configPopUp);
}

void  gui_ok_conf_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   gui_autosave_time_CB(autosavetimeText, NULL, NULL);

   XtUnmanageChild(configPopUp);
   if ((WordWrap != wordWrap) || (AutoIndent != autoIndent))
      set_autoindent_wordwrap();
   if (AutoSave != autoSave)
      set_autosave();
}

void  gui_autosave_time_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   char     txt[20];
   char    *temp;

   temp = XmTextGetString(autosavetimeText);
   AutoSaveTime = atoi(temp);
   if (temp != NULL)
      XtFree(temp);
   if (AutoSaveTime < MIN_AUTO_SAVE_TIME)
   {
      AutoSaveTime = MIN_AUTO_SAVE_TIME;
      sprintf(txt, "%d", AutoSaveTime);
      XmTextSetString(autosavetimeText, txt);
   }
}


void  gui_toggle_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   int *p = (int*)client_data;
   *p = ((XmToggleButtonCallbackStruct *)call_data)->set;
}

