#include <stdio.h>
#include <string.h>

#include <Xm/Xm.h>
#include <Xm/BulletinB.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/List.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleB.h>

#include "sys_consts.h"
#include "globals.h"
#include "callbacks.h"
#include "helpevents.h"
#include "pop_ups.h"


/* Globals */
Widget   configPopUp = (Widget)0;
Widget   autosavetimeText;
Widget   autosaveTB, backupTB, autoindTB, wordwrapTB, fileboxTB;
Widget   helpPopUp = (Widget)0;
Widget   helpLI, helpText;


void  gui_create_help_popup(Widget parent)
{
   Arg      args[15];
   int      n;
   Widget   helpFO, helpControlFO, helpListFO, helpLabel, button;


   helpPopUp = XmCreateBulletinBoardDialog(parent, "Help", NULL, 0);

   helpFO = XtVaCreateWidget("helpFO", xmFormWidgetClass, helpPopUp, NULL);

   helpListFO = XtVaCreateWidget("helpListFO", xmFormWidgetClass, helpFO,
			XmNleftAttachment,   XmATTACH_FORM,
			XmNtopAttachment,    XmATTACH_FORM,
			XmNbottomAttachment, XmATTACH_FORM,
			NULL);

   helpLabel = XtVaCreateManagedWidget("topics", xmLabelWidgetClass, helpListFO,
			XmNleftAttachment,  XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_FORM,
			XmNtopAttachment,   XmATTACH_FORM,
			NULL);

   n = 0;
   XtSetArg(args[n], XmNvisibleItemCount, 20);              n++;
   XtSetArg(args[n], XmNlistSizePolicy,   XmCONSTANT);      n++;
   XtSetArg(args[n], XmNselectionPolicy,  XmBROWSE_SELECT); n++;
   XtSetArg(args[n], XmNleftAttachment,   XmATTACH_FORM);   n++;
   XtSetArg(args[n], XmNrightAttachment,  XmATTACH_FORM);   n++;
   XtSetArg(args[n], XmNtopWidget,        helpLabel);       n++;
   XtSetArg(args[n], XmNtopAttachment,    XmATTACH_WIDGET); n++;
   helpLI = XmCreateScrolledList(helpListFO, "helpLI", args, n);
   XtAddCallback(helpLI, XmNbrowseSelectionCallback, gui_help_sel_CB, NULL);
   XtManageChild(helpLI);

   helpControlFO = XtVaCreateWidget("control", xmRowColumnWidgetClass, helpListFO,
			XmNnumColumns,      2,
			XmNorientation,     XmHORIZONTAL,
			XmNtopWidget,       helpLI,
			XmNtopAttachment,   XmATTACH_WIDGET,
			XmNrightAttachment, XmATTACH_FORM,
			XmNleftAttachment,  XmATTACH_FORM,
			NULL);

   button = XtVaCreateManagedWidget("Help", xmPushButtonGadgetClass, helpControlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_help_CB, (XtPointer)HELPHELP);

   button = XtVaCreateManagedWidget("cancel", xmPushButtonGadgetClass, helpControlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_help_CB, (XtPointer) HELPCANCEL);

   XtManageChild(helpControlFO);
   XtManageChild(helpListFO);

   n = 0;
   XtSetArg(args[n], XmNeditable,              False);             n++;
   XtSetArg(args[n], XmNeditMode,              XmMULTI_LINE_EDIT); n++;
   XtSetArg(args[n], XmNcursorPositionVisible, False);             n++;
   XtSetArg(args[n], XmNleftAttachment,        XmATTACH_WIDGET);   n++;
   XtSetArg(args[n], XmNleftWidget,            helpListFO);        n++;
   XtSetArg(args[n], XmNrightAttachment,       XmATTACH_FORM);     n++;
   XtSetArg(args[n], XmNtopAttachment,         XmATTACH_FORM);     n++;
   XtSetArg(args[n], XmNbottomAttachment,      XmATTACH_FORM);     n++;
   helpText = XmCreateScrolledText(helpFO, "helpText", args, n);
   XtManageChild(helpText);

   gui_create_helplist();

   XtManageChild(helpFO);
}


void     gui_create_config_popup(Widget parent)
{
   char     txt[20];
   Arg      args[15];
   int      n;
   Widget   autosavetimeFrame, onOffFrame, controlFrame,
            autosavetimeFO, CheckLabel, controlFO, configFO, button;
   Widget   optionCheck;


   configPopUp = XmCreateBulletinBoardDialog(parent, "Configuration", NULL, 0);

   configFO = XtVaCreateWidget("configFO", xmFormWidgetClass, configPopUp, NULL);

   onOffFrame = XtVaCreateWidget("onOff", xmFrameWidgetClass, configFO,
			XmNtopAttachment,  XmATTACH_FORM,
			XmNleftAttachment, XmATTACH_FORM,
			NULL);

   controlFrame = XtVaCreateWidget("control", xmFrameWidgetClass, configFO,
			XmNtopAttachment,   XmATTACH_FORM,
			XmNleftAttachment,  XmATTACH_WIDGET,
			XmNleftWidget,      onOffFrame,
			XmNleftOffset,      5,
			XmNrightAttachment, XmATTACH_FORM,
			NULL);

   autosavetimeFrame = XtVaCreateWidget("autosavetime", xmFrameWidgetClass, configFO,
			XmNrightAttachment, XmATTACH_FORM,
			XmNtopAttachment,   XmATTACH_WIDGET,
			XmNtopWidget,       controlFrame,
			XmNtopOffset,       5,
			XmNleftAttachment,  XmATTACH_FORM,
			NULL);

   autosavetimeFO = XtVaCreateWidget("Time", xmRowColumnWidgetClass, autosavetimeFrame,
			XmNnumColumns,  3,
			XmNorientation, XmHORIZONTAL,
			XmNspacing,     3,
			NULL);

   XtVaCreateManagedWidget("asave_time1", xmLabelGadgetClass, autosavetimeFO, NULL);

   sprintf(txt, "%d", AutoSaveTime);
   autosavetimeText = XtVaCreateManagedWidget("autosavetimeText", xmTextFieldWidgetClass, autosavetimeFO,
			XmNvalue,     txt,
			XmNmaxLength, 20,
			XmNcolumns,   5,
			NULL);
   XtAddCallback(autosavetimeText, XmNlosingFocusCallback, gui_autosave_time_CB, NULL);

   XtVaCreateManagedWidget("AutosaveTime(seconds)", xmLabelGadgetClass, autosavetimeFO, NULL);

   XtManageChild(autosavetimeFO);
   XtManageChild(autosavetimeFrame);

   CheckLabel = XtVaCreateManagedWidget("CheckLabel", xmLabelGadgetClass, onOffFrame,
			XmNchildType, XmFRAME_TITLE_CHILD,
			NULL);

   n = 0;
   XtSetArg(args[n], XmNnumColumns, 1); n++;
   XtSetArg(args[n], XmNspacing,    3); n++;
   optionCheck = XmCreateSimpleCheckBox(onOffFrame, "optionCheck", args, n);

   autosaveTB = XtVaCreateManagedWidget("Autosave", xmToggleButtonWidgetClass, optionCheck, NULL);
   XtAddCallback(autosaveTB, XmNvalueChangedCallback, gui_toggle_CB, (XtPointer)&AutoSave);

   backupTB = XtVaCreateManagedWidget("Backup Files", xmToggleButtonWidgetClass, optionCheck, NULL);
   XtAddCallback(backupTB, XmNvalueChangedCallback, gui_toggle_CB, (XtPointer)&AutoBackup);

   autoindTB = XtVaCreateManagedWidget("Auto-Indent", xmToggleButtonWidgetClass, optionCheck, NULL);
   XtAddCallback(autoindTB, XmNvalueChangedCallback, gui_toggle_CB, (XtPointer)&AutoIndent);

   wordwrapTB = XtVaCreateManagedWidget("wordwrap", xmToggleButtonWidgetClass, optionCheck, NULL);
   XtAddCallback(wordwrapTB, XmNvalueChangedCallback, gui_toggle_CB, (XtPointer)&WordWrap);

   fileboxTB = XtVaCreateManagedWidget("filebox", xmToggleButtonWidgetClass, optionCheck, NULL);
   XtAddCallback(fileboxTB, XmNvalueChangedCallback, gui_toggle_CB, (XtPointer)&FileBox);

   XtManageChild(optionCheck);
   XtManageChild(onOffFrame);

   controlFO = XtVaCreateWidget("controlbox", xmRowColumnWidgetClass, controlFrame,
			XmNnumColumns,     1,
			XmNpacking,        XmPACK_COLUMN,
			XmNentryAlignment, XmALIGNMENT_CENTER,
			XmNisAligned,      True,
			NULL);

   button = XtVaCreateManagedWidget("OK", xmPushButtonGadgetClass, controlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_ok_conf_CB, NULL);

   button = XtVaCreateManagedWidget("Save", xmPushButtonGadgetClass, controlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_save_conf_CB, NULL);

   button = XtVaCreateManagedWidget("Cancel", xmPushButtonGadgetClass, controlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_cancel_conf_CB, NULL);

   button = XtVaCreateManagedWidget("Restore", xmPushButtonGadgetClass, controlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_restore_conf_CB, NULL);

   button = XtVaCreateManagedWidget("Help", xmPushButtonGadgetClass, controlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_help_CB, (XtPointer)HELPSETUP);

   XtManageChild(controlFO);

   XtManageChild(controlFrame);

   XtManageChild(configFO);
}

