#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <Xm/Xm.h>
#include <Xm/Text.h>

#include "sys_consts.h"
#include "globals.h"
#include "utils.h"
#include "callbacks.h"

#include "heap_string.h"

Boolean  SelectedOnly;

static int  gui_search(void);
static int  determine_row_height(void);
static void replace_selection(char *word, char *replacement);
static void replaceText(char *word, char *replacement);

static int  determine_row_height(void)
{
   int  base = XmTextGetBaseline (text_canvas);
   Dimension s, m, h;
   XtVaGetValues(text_canvas,
		XmNmarginHeight,      &m,
		XmNshadowThickness,   &s,
		XmNhighlightThickness, &h,
		NULL);
   return(base - s - h - m + 1);
}

void  gui_jumpto_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   char      *text;
   char      *temp;
   int        linenum;
   int        linecount;
   int        charcount, charcount2;
   int        textlength;
   Dimension  height;
   int        rows;

   linecount = 1;
   charcount = charcount2 = -1;

   temp = XmTextGetString(jumpMenuText);
   linenum = -1;
   if (temp)
   {
      if (*temp)
         linenum = atoi(temp);
      XtFree(temp);
   }
   if (linenum < 0)
      return;

   text = XmTextGetString(text_canvas);
   textlength = strlen(text);
   XtVaGetValues(text_canvas, XmNheight, &height, NULL);
   rows = (int)height / determine_row_height();

   while ((charcount < textlength) && (linecount < linenum))
   {
      charcount++;
      if (text[charcount] == '\n')
	 linecount++;
      if (linecount == linenum - (int)rows / 2)
	 charcount2 = charcount + 1;
   };
   charcount++;

   if (linecount != linenum)
   {
      set_status_message("sorry line number invalid");
   }
   else
   {
      XmTextSetTopCharacter(text_canvas, (XmTextPosition)charcount2);
      XmTextSetInsertionPosition(text_canvas, (XmTextPosition)charcount);
      set_status_message("Ok jumping to line");
   }

   XtFree(text);
}

void  gui_search_repl_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   switch ((uintptr_t)client_data)
   {
      case SEARCH:
	 set_cursor(toplevel, TimeCursor);
	 set_cursor(text_canvas, TimeCursor);
	 if (gui_search() > 0)
	 {
	    set_status_message("Found an instance");

	    XtSetSensitive(replaceMenuPB, True);
	 }
	 else
	    set_status_message("Sorry couldn't find the word");

	 set_cursor(toplevel, MainCursor);
	 set_cursor(text_canvas, TextCursor);
	 break;

      case REPLACE:		
	 gui_replace_it_CB((Widget) 0, NULL, NULL);
	 XtSetSensitive(replaceMenuPB, False);
	 break;

      case REPLACEALL:		
	 set_cursor(toplevel, TimeCursor);
	 set_cursor(text_canvas, TimeCursor);
	 gui_replace_all_it_CB((Widget) 0, NULL, NULL);
	 XtSetSensitive(replaceMenuPB, False);
	 set_cursor(toplevel, MainCursor);
	 set_cursor(text_canvas, TextCursor);
	 break;
   }
}

void  search_selectiononly_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   XtVaGetValues(w, XmNset, &SelectedOnly, NULL);
}

static int  gui_search(void)
{
   char     *text = NULL;
   char     *base = NULL;
   char     *word = NULL;
   char     *instanceofwordptr = NULL;
   int       pos, wordpos, rows, linecount, charcount;
   Dimension height;

   word = XmTextGetString(searchMenuText);
   if (!word || !*word)
   {
      XtFree(word);
      return(0);
   }

   pos = XmTextGetInsertionPosition(text_canvas);
   base = XmTextGetString(text_canvas);
   text = base + pos;

   instanceofwordptr = strstr(text, word);
   if (instanceofwordptr == NULL)
   {
      instanceofwordptr = strstr(base, word);
      if (instanceofwordptr == NULL)
      {
	 XtFree(base);
	 XtFree(word);
	 return(0);
      }
   }

   wordpos = (instanceofwordptr - base);

   XtVaGetValues(text_canvas, XmNheight, &height, NULL);
   rows = (int)height / determine_row_height();

   charcount = wordpos;
   linecount = (int)rows / 2;
   while ((linecount > 0) && (charcount > 1))
   {
      charcount--;
      if (base[charcount] == '\n')
         linecount--;
   }
   XmTextSetTopCharacter(text_canvas, charcount);

   XmTextSetInsertionPosition(text_canvas, (XmTextPosition)(wordpos + strlen(word)));

   XmTextSetSelection(text_canvas, (XmTextPosition)wordpos, (XmTextPosition)(wordpos + strlen(word)), CurrentTime);

   XtFree(word);
   XtFree(base);

   return(1);
}

void  gui_replace_it_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   char    *word;
   char    *replacement;
   char    *temp;
   XmTextPosition pos, firstpos;

   word = XmTextGetString(searchMenuText);
   replacement = XmTextGetString(replaceMenuText);
   temp = XmTextGetSelection(text_canvas);
   if (strcmp(temp, word) == 0)
   {
      XmTextGetSelectionPosition(text_canvas, &firstpos, &pos);

      XmTextReplace(text_canvas, firstpos, pos, replacement);

      XmTextSetInsertionPosition(text_canvas, (XmTextPosition) (firstpos + strlen(replacement)));

      XtSetSensitive(replaceMenuPB, False);
   }
   else
      set_status_message("Sorry can't replace, ...you lost the selection");

   XtFree(word);
   XtFree(replacement);
   XtFree(temp);
}


void  gui_replace_all_it_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   char    *word;
   char    *replacement;

   word = XmTextGetString(searchMenuText);
   replacement = XmTextGetString(replaceMenuText);

   if (!*word)
      set_status_message("Sorry No search text specified");
   else
   {
      if (strcmp(word, replacement) != 0)
      {
         if (SelectedOnly)
	    replace_selection(word, replacement);
	 else
	    replaceText(word, replacement);
      }
      else
          set_status_message("They're the same why bother");
   }

   XtFree(word);
   XtFree(replacement);
}

static void replace_selection(char *word, char *replacement)
{
   char    *temp;
   char    *temp1;
   hstring  final;
   char    *instanceptr;
   XmTextPosition left, right;

   heap_string_init(&final);
   temp = XmTextGetSelection(text_canvas);
   if (temp && *temp)
   {
      set_status_message("Ok replacing instances within selection");
      XmTextGetSelectionPosition(text_canvas, &left, &right);
      temp1 = temp;

      /* Now do search and replace on string */
      instanceptr = strstr(temp1, word);
      if (instanceptr != NULL)
      {
         while (instanceptr != NULL)
	 {
	    heap_strncat(&final, temp1, instanceptr - temp1);
	    heap_string_cat(&final, replacement);
	    temp1 = instanceptr + strlen(word);
	    instanceptr = strstr(temp1, word);
	 }
         heap_string_cat(&final, temp1);
         XmTextReplace(text_canvas, left, right, final.str);
         XmTextSetSelection(text_canvas, left, left + strlen(final.str), CurrentTime);
         set_status_message("Done replacing instances within selection");
      }
      else
      {
         set_status_message("Sorry no instances found");
      }
   }
   else
   {
      set_status_message("Sorry no selection to edit");
   }
   XtFree(temp);
   heap_string_free(&final);
}

static void replaceText(char *word, char *replacement)
{
   char    *temp;
   char    *temp1;
   hstring  final;
   char    *instanceptr;

   heap_string_init(&final);
   set_status_message("Ok replacing instances");
   temp = XmTextGetString(text_canvas);
   temp1 = temp;
   instanceptr = strstr(temp1, word);
   if (instanceptr != NULL)
   {
      while (instanceptr != NULL)
      {
	 heap_strncat(&final, temp1, instanceptr - temp1);
	 heap_string_cat(&final, replacement);
	 temp1 = instanceptr + strlen(word);
	 instanceptr = strstr(temp1, word);
      }
      heap_string_cat(&final, temp1);
      XmTextSetString(text_canvas, final.str);
      set_status_message("Done replacing instances");
   }
   else
   {
      set_status_message("Sorry no instances found");
   }
   XtFree(temp);
   heap_string_free(&final);
}

void  gui_match_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   char    *parentheses;
   char    *temp;
   int      matchChar;
   int      up, down, index;
   XmTextPosition left, right;

   parentheses = XmTextGetSelection(text_canvas);
   temp = XmTextGetString(text_canvas);
   XmTextGetSelectionPosition(text_canvas, &left, &right);

   if (parentheses != NULL)
   {
      if (strlen(parentheses) == 1)
      {
	 matchChar = '\0';
	 set_status_message("Ok matching");
	 if (parentheses[0] == '(')
	    matchChar = ')';
	 if (parentheses[0] == '{')
	    matchChar = '}';
	 if (parentheses[0] == '<')
	    matchChar = '>';
	 if (parentheses[0] == ')')
	    matchChar = '(';
	 if (parentheses[0] == '}')
	    matchChar = '{';
	 if (parentheses[0] == '>')
	    matchChar = '<';

	 if ((matchChar == ')') || (matchChar == '}') || (matchChar == '>'))
	 {
	    index = (int)left + 1;
	    up = 1;
	    down = 0;
	    while (up != down && (index <= strlen(temp) - 1))
	    {
	       if (temp[index] == parentheses[0])
		  up++;
	       if (temp[index] == matchChar)
		  down++;
	       index++;
	    }
	    if (up == down)
	       XmTextSetSelection(text_canvas, (XmTextPosition) left, (XmTextPosition) index, CurrentTime);
	    else
	       set_status_message("Sorry! No match found");
	 }
	 else
	 {
	    if ((matchChar == '(') || (matchChar == '{') || (matchChar == '<'))
	    {
	       index = (int)left - 1;
	       up = 1;
	       down = 0;
	       while (up != down && (index >= 0))
	       {
		  if (temp[index] == parentheses[0])
		     up++;
		  if (temp[index] == matchChar)
		     down++;
		  index--;
	       }
	       if (up == down)
		  XmTextSetSelection(text_canvas, (XmTextPosition) index + 1, (XmTextPosition) left + 1, CurrentTime);
	       else
		  set_status_message("Sorry! No match found");
	    }
	    else
	       set_status_message("Sorry! No match found");
	 }
      }
      else
	 set_status_message("Sorry! The selection is ambigous");
   }
   XtFree(temp);
   XtFree(parentheses);
}
