/* SiME utilities code */

/* Copyright 2021 Kazuo K.
 * SiME is licensed under the MIT license. You should have
 * received a copy of the license with your distribution.
 */
#include <string.h>
#include <stdlib.h>

#include <X11/cursorfont.h>
#include <Xm/Xm.h>
#include <Xm/MessageB.h>
#include <Xm/SelectioB.h>
#include <Xm/FileSB.h>
#include <Xm/Text.h>
#include <Xm/PushB.h>

#include "sys_consts.h"
#include "globals.h"
#include "callbacks.h"
#include "utils.h"
#include "heap_string.h"


/***  VARIABLE DECLARATIONS  ***/
static int   button_pressed;


/***  PROCEDURE DECLARATIONS  ***/
static void  gui_errorCB(Widget w, XtPointer client_data, XtPointer call_data);
static void  gui_query_CB(Widget w, XtPointer client_data, XtPointer call_data);



void  gui_info(char *message)
{
   static Widget  info_DI;
   XmString       msg;

   if (info_DI == (Widget)0)
   {
      info_DI = XmCreateInformationDialog(toplevel, "InfoBox", NULL, 0);

      /* now turn off unwanted buttons */
      XtUnmanageChild(XmMessageBoxGetChild(info_DI, XmDIALOG_HELP_BUTTON));
      XtUnmanageChild(XmMessageBoxGetChild(info_DI, XmDIALOG_CANCEL_BUTTON));
   }

   msg = XmStringCreateLtoR(message, CHARSET);
   XtVaSetValues(info_DI, XmNmessageString, msg, NULL);
   XmStringFree(msg);

   XBell(XtDisplay(info_DI), BELL_VOL);
   XtManageChild(info_DI);
}


int  gui_error(char *message)
{
   static Widget  error_DI;
   XEvent         app_event;
   XmString       msg;

   if (error_DI == (Widget)0)
   {
      error_DI = XmCreateErrorDialog(toplevel, "ErrorBox", NULL, 0);

      XtAddCallback(error_DI, XmNokCallback, gui_errorCB, (XtPointer)YES);
      XtAddCallback(error_DI, XmNhelpCallback, gui_errorCB, (XtPointer)HELP);

      /* now turn off unwanted buttons */
      XtUnmanageChild(XmMessageBoxGetChild(error_DI, XmDIALOG_CANCEL_BUTTON));
   }

   msg = XmStringCreateLtoR(message, CHARSET);
   XtVaSetValues(error_DI, XmNmessageString, msg, NULL);
   XmStringFree(msg);

   XBell(XtDisplay(error_DI), BELL_VOL);
   XtManageChild(error_DI);

   button_pressed = NOTHING;
   while (button_pressed == NOTHING)
   {
      XtAppNextEvent(app_context, &app_event);
      XtDispatchEvent(&app_event);
   }
   XtUnmanageChild(error_DI);
   return button_pressed;
}

static void  gui_errorCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   button_pressed = (uintptr_t)client_data;
}


char  gui_query(char *message)
{
   static Widget  query_DI;
   XEvent         app_event;
   XmString       msg;

   if (query_DI == (Widget) 0)
   {
      Arg      args[5];
      int      n = 0;
      Widget   save_button;

      XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL);
      n++;
      query_DI = XmCreateQuestionDialog(toplevel, "queryDI", args, n);

      save_button = XtVaCreateManagedWidget("Save", xmPushButtonWidgetClass, query_DI, NULL);
      XtAddCallback(save_button, XmNactivateCallback, gui_query_CB, (XtPointer)SAVE_THEN_DO);

      XtAddCallback(query_DI, XmNokCallback, gui_query_CB, (XtPointer)YES);
      XtAddCallback(query_DI, XmNcancelCallback, gui_query_CB, (XtPointer)NO);

      XtUnmanageChild(XmMessageBoxGetChild(query_DI, XmDIALOG_HELP_BUTTON));
   }

   msg = XmStringCreateLtoR(message, CHARSET);
   XtVaSetValues(query_DI, XmNmessageString, msg, NULL);
   XmStringFree(msg);

   XBell(XtDisplay(query_DI), BELL_VOL);
   XtManageChild(query_DI);

   button_pressed = NOTHING;
   while (button_pressed == NOTHING)
   {
      XtAppNextEvent(app_context, &app_event);
      XtDispatchEvent(&app_event);
   }
   return(button_pressed);
}

static void  gui_query_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
   button_pressed = (uintptr_t)client_data;
}



void  set_cursor(Widget w, Cursor c)
{
   XDefineCursor(XtDisplay(w), XtWindow(w), c);
   XFlush(XtDisplay(w));
}


void  unset_cursor(Widget w)
{
   XUndefineCursor(XtDisplay(w), XtWindow(w));
   XFlush(XtDisplay(toplevel));
}


/* Change the title message */
void  set_title_string(void)
{
   hstring  titleMessage;
   char    *name;

   name = strrchr(FileName, '/');
   if (name) name++;
   else      name = FileName;

   heap_string_init(&titleMessage);

   /* First put the progeam nam in at the front */
   heap_string_cat(&titleMessage, "SiME - <");

   /* Now put the file name in */
   heap_string_cat(&titleMessage, name);
   if (changes_counter)
      heap_string_cat(&titleMessage, "*");
   heap_string_cat(&titleMessage, ">");

   XtVaSetValues(toplevel, XmNtitle, titleMessage.str, NULL);

   /* Make sure the users sees the message straight away */
   XFlush(XtDisplay(toplevel));

   heap_string_free(&titleMessage);
}

/* Change the help message in the status line */
void  set_status_message(char *msg)
{
   XmString str = XmStringCreateLtoR(msg, CHARSET);
   XtVaSetValues(statusBar, XmNlabelString, str, NULL);
   XmStringFree(str);
}
