/* SiME Widget creation code */

/* Copyright 2021 Kazuo K.
 * SiME is licensed under the MIT license. You should have
 * received a copy of the license with your distribution.
 */

#include <stdio.h>
#include <string.h>

#include <Xm/Xm.h>
#include <Xm/AtomMgr.h>
#include <Xm/Protocols.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/LabelG.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleB.h>
#include <Xm/CascadeB.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/Separator.h>
#include <Xm/LabelG.h>

#include "sys_consts.h"
#include "globals.h"
#include "widgets.h"
#include "callbacks.h"

/* Globals */
Widget   text_canvas;
Widget   statusBar;
Widget   undoMenuPB, redoMenuPB, replaceMenuPB;
Widget   jumpMenuText;
Widget   searchMenuText, replaceMenuText;

/* Locals */
static Widget  toolbarw;
static Widget  main_fo;

static void gui_create_toolbar(Widget parent);

/* Build our widgets */
void     gui_create_Widgets(Widget toplevel) {
    Atom     WM_DEL;
    Widget   s_frame;

/* react on the close command from the window manager menu */
    WM_DEL = XmInternAtom(XtDisplay(toplevel), "WM_DELETE_WINDOW", False);
    XmAddWMProtocolCallback(toplevel, WM_DEL, gui_quit_CB, (XtPointer)NULL);

   main_fo = XtVaCreateWidget("main_fo", xmFormWidgetClass, toplevel, NULL);

   toolbarw = XtVaCreateWidget("toolbarw", xmRowColumnWidgetClass, main_fo,
			XmNtopAttachment,   XmATTACH_FORM,
			XmNleftAttachment,  XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_FORM,
			XmNresizeHeight,    True,
			XmNspacing,         3,
			XmNadjustLast,      True,
			NULL);
   gui_create_toolbar(toolbarw);
   XtManageChild(toolbarw);

/* status bar */
   s_frame = XtVaCreateWidget("statusFrame", xmFrameWidgetClass, main_fo,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNleftAttachment,   XmATTACH_FORM,
			XmNrightAttachment,  XmATTACH_FORM,
			NULL);
   statusBar = XtVaCreateManagedWidget("statusBar", xmLabelGadgetClass, s_frame, NULL);
   XtManageChild(s_frame);

/* Editor Space */
   create_text_canvas();

   XtManageChild(main_fo);
}


void create_text_canvas(void)
{
   Arg      args[20];
   int      n;
   static XmTextScanType sarray[] = {
      XmSELECT_POSITION, XmSELECT_WORD,
      XmSELECT_LINE,     XmSELECT_PARAGRAPH,
      XmSELECT_ALL
   };

   n = 0;
   XtSetArg(args[n], XmNeditMode,            XmMULTI_LINE_EDIT); n++;
   XtSetArg(args[n], XmNselectionArrayCount, 5);                 n++;
   XtSetArg(args[n], XmNselectionArray,      sarray);            n++;
   XtSetArg(args[n], XmNtopAttachment,       XmATTACH_WIDGET);   n++;
   XtSetArg(args[n], XmNtopWidget,           toolbarw);           n++;
   XtSetArg(args[n], XmNbottomAttachment,    XmATTACH_WIDGET);   n++;
   XtSetArg(args[n], XmNbottomWidget,        statusBar);         n++;
   XtSetArg(args[n], XmNleftAttachment,      XmATTACH_FORM);     n++;
   XtSetArg(args[n], XmNrightAttachment,     XmATTACH_FORM);     n++;
   XtSetArg(args[n], XmNwordWrap,            WordWrap);          n++;
   XtSetArg(args[n], XmNscrollHorizontal,    False);             n++;

   text_canvas = XmCreateScrolledText(main_fo, "text_canvas", args, n);
   XtAddCallback(text_canvas, XmNmodifyVerifyCallback, gui_keypress_CB, NULL);
   XtManageChild(text_canvas);
}

static void installHelpCBs(Widget w, XtCallbackProc p, XtPointer d, uintptr_t id)
{
   XtAddCallback(w, XmNactivateCallback, p, d);
   XtAddCallback(w, XmNarmCallback, gui_cs_help_CB, (XtPointer)id);
   XtAddCallback(w, XmNdisarmCallback, gui_undo_cs_help_CB, (XtPointer)id);
}

static void gui_create_toolbar(Widget parent)
{
   Widget   menuBar;
   Widget   secondBar, secondBarFrame, helpPullDown, searchPullDown;
   Widget   optionPullDown, filePullDown, editPullDown;
   Widget   menuPB;
   Widget   lbl;

/* MENU BAR */
   menuBar = XmCreateMenuBar(parent, "menuBar", NULL, 0);

/* FILE PULLDOWN */
   filePullDown = XmCreatePulldownMenu(menuBar, "filePullDown", NULL, 0);

   XtVaCreateManagedWidget("File", xmCascadeButtonWidgetClass, menuBar,
			   XmNsubMenuId, filePullDown,
			   NULL);

/* NEW BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("New", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_new_CB, NULL, NEW);

/* OPEN BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("Open", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_browse_CB, (XtPointer)OPENIT, OPEN);

/* SAVE BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("Save", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_save_CB, NULL, SAVE);

/* SAVE AS BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("Save As", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_saveas_CB, NULL, SAVEAS);

/* QUIT BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("Quit", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_quit_CB, NULL, QUIT);

/* EDIT PULLDOWN */
   editPullDown = XmCreatePulldownMenu(menuBar, "editPullDown", NULL, 0);

   XtVaCreateManagedWidget("Edit", xmCascadeButtonWidgetClass, menuBar,
			   XmNsubMenuId, editPullDown,
			   NULL);

/* UNDO BUTTON WIDGET */
   undoMenuPB = XtVaCreateManagedWidget("Undo", xmPushButtonGadgetClass, editPullDown, NULL);
   installHelpCBs(undoMenuPB, gui_undo_CB, (XtPointer)UNDO, UNDO);

/* REDO BUTTON WIDGET */
   redoMenuPB = XtVaCreateManagedWidget("Redo", xmPushButtonGadgetClass, editPullDown, NULL);
   installHelpCBs(redoMenuPB, gui_undo_CB, (XtPointer)REDO, REDO);

   XtVaCreateManagedWidget("Separator", xmSeparatorWidgetClass, editPullDown, NULL);

/* COPY BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("Copy", xmPushButtonGadgetClass, editPullDown, NULL);
   installHelpCBs(menuPB, cut_copy_CB, (XtPointer)1, COPY);

/* CUT BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("Cut", xmPushButtonGadgetClass, editPullDown, NULL);
   installHelpCBs(menuPB, cut_copy_CB, (XtPointer)0, CUT);

/* PASTE BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("Paste", xmPushButtonGadgetClass, editPullDown, NULL);
   installHelpCBs(menuPB, gui_paste_CB, NULL, PASTE);

/* CREATE SEARCH PULL DOWN */
   searchPullDown = XmCreatePulldownMenu(menuBar, "searchPullDown", NULL, 0);

   XtVaCreateManagedWidget("Search", xmCascadeButtonWidgetClass, menuBar,
			   XmNsubMenuId, searchPullDown,
			   NULL);

/* SEARCH BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("Search For", xmPushButtonGadgetClass, searchPullDown, NULL);
   installHelpCBs(menuPB, gui_search_repl_CB, (XtPointer)SEARCH, SEARCH);

/* REPLACE BUTTON WIDGET */
   replaceMenuPB = XtVaCreateManagedWidget("Replace", xmPushButtonGadgetClass, searchPullDown, NULL);
   installHelpCBs(replaceMenuPB, gui_search_repl_CB, (XtPointer)REPLACE, REPLACE);
   XtSetSensitive(replaceMenuPB, False);

/* REPLACE ALL BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("Replace All", xmPushButtonGadgetClass, searchPullDown, NULL);
   installHelpCBs(menuPB, gui_search_repl_CB, (XtPointer)REPLACEALL, REPLACEALL);

/* TOGGLE SELECTION BUTTON */
   menuPB = XtVaCreateManagedWidget("Toggle Selection", xmToggleButtonWidgetClass, searchPullDown,
			XmNset, False,
			NULL);
   XtAddCallback(menuPB, XmNvalueChangedCallback, search_selectiononly_CB, NULL);

/* MATCH BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("Match", xmPushButtonGadgetClass, searchPullDown, NULL);
   installHelpCBs(menuPB, gui_match_CB, NULL, MATCH);

/* CREATE OPTION PULL DOWN */
   optionPullDown = XmCreatePulldownMenu(menuBar, "optionPullDown", NULL, 0);
   XtVaCreateManagedWidget("Options", xmCascadeButtonWidgetClass, menuBar,
			XmNsubMenuId, optionPullDown,
			NULL);

/* setUp BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("Setup", xmPushButtonGadgetClass, optionPullDown, NULL);
   installHelpCBs(menuPB, gui_setup_CB, NULL, SETUP);

/* CREATE HELP PULL DOWN */
   helpPullDown = XmCreatePulldownMenu(menuBar, "helpPullDown", NULL, 0);
   menuPB =XtVaCreateManagedWidget("Help", xmCascadeButtonWidgetClass, menuBar,
			XmNsubMenuId, helpPullDown,
			NULL);
   XtVaSetValues(menuBar, XmNmenuHelpWidget,menuPB, NULL);

/* HELP BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("Help", xmPushButtonGadgetClass, helpPullDown, NULL);
   installHelpCBs(menuPB, gui_help_CB, (XtPointer)HELPCALL, HELPB);

   menuPB = XtVaCreateManagedWidget("About", xmPushButtonGadgetClass, helpPullDown, NULL);
   XtAddCallback(menuPB, XmNactivateCallback, gui_help_CB, (XtPointer)HELPABOUT);

   XtManageChild(menuBar);


/* CREATE FRAME FOR SECOND BAR AREA */
   secondBarFrame = XtVaCreateWidget("secondBarFrame", xmFrameWidgetClass, toolbarw, NULL);

/* BUTTON BAR WIDGET */
   secondBar = XtVaCreateWidget("secondBar", xmFormWidgetClass, secondBarFrame, NULL);

/* JUMP BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("Jump", xmPushButtonWidgetClass, secondBar,
				XmNtopAttachment,    XmATTACH_FORM,
				XmNleftAttachment,   XmATTACH_FORM,
				XmNbottomAttachment, XmATTACH_FORM,
				NULL);
   installHelpCBs(menuPB, gui_jumpto_CB, NULL, JUMP);

/* JUMP MenuText Widget */
   jumpMenuText = XtVaCreateManagedWidget("jumpMenuText", xmTextFieldWidgetClass, secondBar,
				XmNmaxLength,        8,
				XmNcolumns,          8,
				XmNtopAttachment,    XmATTACH_FORM,
				XmNleftAttachment,   XmATTACH_WIDGET,
				XmNleftWidget,       menuPB,
				XmNbottomAttachment, XmATTACH_FORM,
				NULL);
   XtAddCallback(jumpMenuText, XmNactivateCallback, gui_jumpto_CB, NULL);
   XtAddCallback(jumpMenuText, XmNfocusCallback, gui_cs_help_CB, (XtPointer)JUMPTEXT);
   XtAddCallback(jumpMenuText, XmNlosingFocusCallback, gui_undo_cs_help_CB, (XtPointer)JUMPTEXT);

/* SEARCH LABEL WIDGET */
   lbl = XtVaCreateManagedWidget("Search", xmLabelGadgetClass, secondBar,
				XmNtopAttachment,    XmATTACH_FORM,
				XmNleftAttachment,   XmATTACH_WIDGET,
				XmNleftWidget,       jumpMenuText,
				XmNbottomAttachment, XmATTACH_FORM,
				NULL);

/* SEARCH MenuText Widget */
   searchMenuText = XtVaCreateManagedWidget("searchMenuText", xmTextFieldWidgetClass, secondBar,
				XmNmaxLength,        100,
				XmNcolumns,          17,
				XmNtopAttachment,    XmATTACH_FORM,
				XmNleftAttachment,   XmATTACH_WIDGET,
				XmNleftWidget,       lbl,
				XmNbottomAttachment, XmATTACH_FORM,
				NULL);
   XtAddCallback(searchMenuText, XmNactivateCallback, gui_search_repl_CB, (XtPointer)SEARCH);
   XtAddCallback(searchMenuText, XmNfocusCallback, gui_cs_help_CB, (XtPointer)SEARCHTEXT);
   XtAddCallback(searchMenuText, XmNlosingFocusCallback, gui_undo_cs_help_CB, (XtPointer)SEARCHTEXT);

/* REPLACE LABEL WIDGET */
   lbl = XtVaCreateManagedWidget("Replace", xmLabelGadgetClass, secondBar,
				XmNtopAttachment,    XmATTACH_FORM,
				XmNleftAttachment,   XmATTACH_WIDGET,
				XmNleftWidget,       searchMenuText,
				XmNbottomAttachment, XmATTACH_FORM,
				NULL);

/* REPLACE MenuText Widget */
   replaceMenuText = XtVaCreateManagedWidget("replaceMenuText", xmTextFieldWidgetClass, secondBar,
				XmNmaxLength,        100,
				XmNcolumns,          17,
				XmNtopAttachment,    XmATTACH_FORM,
				XmNleftAttachment,   XmATTACH_WIDGET,
				XmNleftWidget,       lbl,
				XmNbottomAttachment, XmATTACH_FORM,
				NULL);
   XtAddCallback(replaceMenuText, XmNfocusCallback, gui_cs_help_CB, (XtPointer)REPLACETEXT);
   XtAddCallback(replaceMenuText, XmNlosingFocusCallback, gui_undo_cs_help_CB, (XtPointer)REPLACETEXT);

   XtManageChild(secondBar);
   XtManageChild(secondBarFrame);
}
